
#!/usr/bin/python
# -*- coding: utf-8 -*-


""" Copyright 2012-2021 Smartling, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this work except in compliance with the License.
 * You may obtain a copy of the License in the LICENSE file, or at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import os
import sys
import time, datetime

sys.path += [os.path.abspath('../'), os.path.abspath('../../')]  # allow to import ../smartlingApiSdk.api

import smartlingApiSdk
from smartlingApiSdk.api.JobsApi import JobsApi
from smartlingApiSdk.ProxySettings import ProxySettings
from smartlingApiSdk.Credentials import Credentials

isPython3 =  sys.version_info[:2] >= (3,0)

def assert_equal(a,b, comment=''):
    if a != b :
        err = "Assertion Failed: '%s' != '%s' %s" % (a,b, comment)
        if not isPython3 and type(err) == str:
            err = err.decode('utf-8', 'ignore')
        raise Exception(repr(err))

class testJobsApi(object):

    CODE_SUCCESS_TOKEN = 'SUCCESS'
    ACCEPTED_TOKEN = 'ACCEPTED'

    def tearDown(self):
        print("tearDown", "OK")

    def setUp(self):
        credentials = Credentials() #Gets your Smartling credetnials from environment variables
        self.MY_USER_IDENTIFIER = credentials.MY_USER_IDENTIFIER
        self.MY_USER_SECRET = credentials.MY_USER_SECRET
        self.MY_PROJECT_ID = credentials.MY_PROJECT_ID

        #needed for testProjects
        self.MY_ACCOUNT_UID = credentials.MY_ACCOUNT_UID
        self.MY_LOCALE = credentials.MY_LOCALE

        if self.MY_ACCOUNT_UID == "CHANGE_ME":
            print("can't test projects api call, set self.MY_ACCOUNT_UID or export SL_ACCOUNT_UID=*********")
            return

        useProxy = False
        if useProxy :
            proxySettings = ProxySettings("login", "password", "proxy_host", "proxy_port or None")
        else:
            proxySettings = None

        self.jobs_api = JobsApi(self.MY_USER_IDENTIFIER, self.MY_USER_SECRET, self.MY_PROJECT_ID, proxySettings)

        print("setUp", "OK", "\n")

        # setUp code add-on #
        self.jobname = 'test_job_'+str(int(time.time()))
        self.deleteTestJobs()

    def deleteTestJobs(self):
        response,code = self.jobs_api.getJobsByProject()
        c = 0
        sz = len(response.data.items)
        for job in response.data.items:
            c += 1
            if job['jobName'].startswith('test_job_'):
                uid = job['translationJobUid']
                cres, cstatus = self.jobs_api.cancelJob(uid, 'test reason')
                res, status = self.jobs_api.deleteJob(uid)
                print (c, 'of', sz, uid, cstatus, status)

    def dateTimeStr(self, offset):
        return datetime.datetime.fromtimestamp(time.time()+offset).strftime("%Y-%m-%dT%H:%M:%SZ")

    def checkAssignCustomFieldsToProject(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/custom-fields
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/assignCustomFieldsToProject
        """

        resp, code = self.jobs_api.getAccountCustomFields(self.MY_ACCOUNT_UID)
        self.fieldUid=None
        for fld in resp.data.items:
            if 'python-sdk-test' == fld['fieldName']:
                self.fieldUid = fld['fieldUid']


        CustomFieldAssignmentList=[{"fieldUid":self.fieldUid},]
        res, status = self.jobs_api.assignCustomFieldsToProject(CustomFieldAssignmentList=CustomFieldAssignmentList)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('assignCustomFieldsToProject', 'OK')


    def checkAddJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs
            as curl :  curl -X POST -H "Authorization: Bearer $smartlingToken" -H "Content-Type: application/json" -d "$smartlingJobJSON" https://api.smartling.com/jobs-api/v3/projects/$smartlingProjectId/jobs
            Responses:
                200 : OK
                400 : Validation error during job creation.
            details :  https://api-reference.smartling.com/#operation/addJob
        """
        jobName=self.jobname
        targetLocaleIds=[self.MY_LOCALE,]
        description='testDescription'
        dueDate=self.dateTimeStr(3600*24*30)
        referenceNumber='testReferenceNumber'
        callbackUrl='https://www.callback.com/smartling/job'
        callbackMethod='GET'
        customFields=[{"fieldUid": self.fieldUid, "fieldValue": "Test Field Value"}]
        res, status = self.jobs_api.addJob(jobName=jobName, targetLocaleIds=targetLocaleIds, description=description, dueDate=dueDate, referenceNumber=referenceNumber, callbackUrl=callbackUrl, callbackMethod=callbackMethod, customFields=customFields)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('addJob', 'OK')
        self.test_job_uid = res.data.translationJobUid


    def checkAddLocaleToJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/locales/{targetLocaleId}
            Responses:
                200 : OK
                202 : ACCEPTED
            details :  https://api-reference.smartling.com/#operation/addLocaleToJob
        """
        translationJobUid=self.test_job_uid
        targetLocaleId="zh-TW" #use your other locale here
        syncContent=True
        res, status = self.jobs_api.addLocaleToJob(translationJobUid=translationJobUid, targetLocaleId=targetLocaleId, syncContent=syncContent)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('addLocaleToJob', 'OK')


    def checkAddStringsToJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/strings/add
            Responses:
                200 : OK
                202 : ACCEPTED
                400 : Validation error response
            details :  https://api-reference.smartling.com/#operation/addStringsToJob
        """
        translationJobUid=self.test_job_uid
        hashcodes=["e1159037badca0a2a618f62c50eff1bb", ] # use your string hashcodes list here
        moveEnabled=False
        targetLocaleIds=[self.MY_LOCALE,]
        res, status = self.jobs_api.addStringsToJob(translationJobUid=translationJobUid, hashcodes=hashcodes, moveEnabled=moveEnabled, targetLocaleIds=targetLocaleIds)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('addStringsToJob', 'OK')
        assert_equal(0, res.data.failCount, "addStringsToJob will fail if string was already in other job")


    def checkAddFileToJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/file/add
            Responses:
                200 : OK
                202 : ACCEPTED
                400 : Validation error adding file to a job
                423 : The requested file is currently being processed by another operation. The file will be unlocked after the operation completes.
            details :  https://api-reference.smartling.com/#operation/addFileToJob
        """
        translationJobUid=self.test_job_uid
        fileUri="test_import.xml_2.2.4_1629202583.584802" #use your actual file uri uploaded earielr to Smartling
        targetLocaleIds=[self.MY_LOCALE,]
        res, status = self.jobs_api.addFileToJob(translationJobUid=translationJobUid, fileUri=fileUri, targetLocaleIds=targetLocaleIds)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('addFileToJob', 'OK')


    def checkGetJobFilesList(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/files
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/getJobFilesList
        """
        translationJobUid=self.test_job_uid
        res, status = self.jobs_api.getJobFilesList(translationJobUid=translationJobUid)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getJobFilesList', 'OK')


    def checkGetJobFileProgress(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/file/progress
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/getJobFileProgress
        """
        translationJobUid=self.test_job_uid
        fileUri="test_import.xml_2.2.4_1629202583.584802" #use your actual file uri uploaded earielr to Smartling
        res, status = self.jobs_api.getJobFileProgress(translationJobUid=translationJobUid, fileUri=fileUri)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getJobFileProgress', 'OK')


    def checkAuthorizeJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/authorize
            Responses:
                200 : OK
                400 : Validation error when authorizing a job
            details :  https://api-reference.smartling.com/#operation/authorizeJob
        """
        translationJobUid=self.test_job_uid
        localeWorkflows= [ { "targetLocaleId": "zh-TW", "workflowUid": "748398939979" } ]
        res, status = self.jobs_api.authorizeJob(translationJobUid=translationJobUid, localeWorkflows=localeWorkflows)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('authorizeJob', 'OK')


    def checkModifyScheduleItemsForTranslationJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/schedule
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/modifyScheduleItemsForTranslationJob
        """
        translationJobUid=self.test_job_uid
        schedules= [ { "targetLocaleId": "zh-TW", "workflowStepUid": "7f6126eff318", "dueDate": self.dateTimeStr(3600*24*30)} ]
        res, status = self.jobs_api.modifyScheduleItemsForTranslationJob(translationJobUid=translationJobUid, schedules=schedules)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('modifyScheduleItemsForTranslationJob', 'OK')


    def checkCreateCustomField(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/accounts/{accountUid}/custom-fields
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/createCustomField
        """
        self.jobs_api.httpClient.ignore_errors=True
        accountUid=self.MY_ACCOUNT_UID
        required=False
        type='SHORT_TEXT'
        fieldName='python-sdk-test'
        enabled=True
        searchable=True
        displayToTranslators=True
        options=[]
        defaultValue='default field value'
        description='Custom field example'
        res, status = self.jobs_api.createCustomField(accountUid=self.MY_ACCOUNT_UID, required=required, type=type, fieldName=fieldName, enabled=enabled, searchable=searchable, displayToTranslators=displayToTranslators, options=options, defaultValue=defaultValue, description=description)


        if 400 == status:
            assert_equal(True, 'Field name must be unique within account' in str(res))
        else:
            assert_equal(True, status in [200,202])
            assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKED])

        print('createCustomField', 'OK')
        self.jobs_api.httpClient.ignore_errors=False


    def checkUpdateCustomField(self):
        """
            method  :  PUT
            api url :  /jobs-api/v3/accounts/{accountUid}/custom-fields/{fieldUid}
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/updateCustomField
        """
        accountUid=self.MY_ACCOUNT_UID
        fieldUid=self.fieldUid
        required=False
        fieldName='python-sdk-test'
        enabled=True
        searchable=True
        displayToTranslators=True
        options=[]
        defaultValue='New default field value'
        description='New custom field example'
        res, status = self.jobs_api.updateCustomField(accountUid=self.MY_ACCOUNT_UID, fieldUid=fieldUid, required=required, fieldName=fieldName, enabled=enabled, searchable=searchable, displayToTranslators=displayToTranslators, options=options, defaultValue=defaultValue, description=description)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('updateCustomField', 'OK')


    def checkRemoveFileFromJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/file/remove
            Responses:
                200 : OK
                202 : ACCEPTED
                404 : Not found validation error
            details :  https://api-reference.smartling.com/#operation/removeFileFromJob
        """
        translationJobUid=self.test_job_uid
        fileUri="test_import.xml_2.2.4_1629202583.584802" #use your actual file uri uploaded earielr to Smartling
        res, status = self.jobs_api.removeFileFromJob(translationJobUid=translationJobUid, fileUri=fileUri)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('removeFileFromJob', 'OK')


    def checkRemoveStringsFromJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/strings/remove
            Responses:
                200 : OK
                202 : ACCEPTED
            details :  https://api-reference.smartling.com/#operation/removeStringsFromJob
        """
        translationJobUid=self.test_job_uid
        hashcodes=["e1159037badca0a2a618f62c50eff1bb", ] # use your string hashcodes list here
        localeIds=[self.MY_LOCALE,]
        res, status = self.jobs_api.removeStringsFromJob(translationJobUid=translationJobUid, hashcodes=hashcodes, localeIds=localeIds)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('removeStringsFromJob', 'OK')


    def checkGetJobLastCompletionDatesPerLocale(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/locales-completion-dates
            Responses:
                200 : OK
                404 : Not found error
            details :  https://api-reference.smartling.com/#operation/getJobLastCompletionDatesPerLocale
        """
        translationJobUid=self.test_job_uid
        res, status = self.jobs_api.getJobLastCompletionDatesPerLocale(translationJobUid=translationJobUid)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getJobLastCompletionDatesPerLocale', 'OK')


    def checkFindScheduleForTranslationJob(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/schedule
            Responses:
                200 : OK
                404 : Not found error
            details :  https://api-reference.smartling.com/#operation/findScheduleForTranslationJob
        """
        translationJobUid=self.test_job_uid
        res, status = self.jobs_api.findScheduleForTranslationJob(translationJobUid=translationJobUid)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('findScheduleForTranslationJob', 'OK')


    def checkRemoveLocaleFromJob(self):
        """
            method  :  DELETE
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/locales/{targetLocaleId}
            Responses:
                200 : OK
                202 : ACCEPTED
            details :  https://api-reference.smartling.com/#operation/removeLocaleFromJob
        """
        translationJobUid=self.test_job_uid
        targetLocaleId="zh-TW" #use already added locale here
        res, status = self.jobs_api.removeLocaleFromJob(translationJobUid=translationJobUid, targetLocaleId=targetLocaleId)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('removeLocaleFromJob', 'OK')


    def checkGetJobsByProject(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs
            as curl :  curl -H "Authorization: Bearer $smartlingToken" https://api.smartling.com/jobs-api/v3/projects/$smartlingProjectId/jobs
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/getJobsByProject
        """
        jobName=self.jobname
        res, status = self.jobs_api.getJobsByProject(jobName=jobName)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getJobsByProject', 'OK')


    def checkUpdateJob(self):
        """
            method  :  PUT
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}
            Responses:
                200 : OK
                400 : Validation error on updating a job
                404 : Job not found error
            details :  https://api-reference.smartling.com/#operation/updateJob
        """
        translationJobUid=self.test_job_uid
        jobName="new name" + self.jobname
        description='new Description'
        dueDate=self.dateTimeStr(3600*24*120)
        referenceNumber='new ReferenceNumber'
        callbackUrl='https://www.callback.com/smartling/new_job'
        callbackMethod='POST'
        customFields=[]
        res, status = self.jobs_api.updateJob(translationJobUid=translationJobUid, jobName=jobName, description=description, dueDate=dueDate, referenceNumber=referenceNumber, callbackUrl=callbackUrl, callbackMethod=callbackMethod, customFields=customFields)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('updateJob', 'OK')


    def checkGetJobProgress(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/progress
            Responses:
                200 : OK
                404 : Not found error
            details :  https://api-reference.smartling.com/#operation/getJobProgress
        """
        translationJobUid=self.test_job_uid
        res, status = self.jobs_api.getJobProgress(translationJobUid=translationJobUid)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getJobProgress', 'OK')


    def checkGetJobDetails(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}
            Responses:
                200 : OK
                404 : Job not found error
            details :  https://api-reference.smartling.com/#operation/getJobDetails
        """
        translationJobUid=self.test_job_uid
        res, status = self.jobs_api.getJobDetails(translationJobUid=translationJobUid)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getJobDetails', 'OK')


    def checkGetStringsForTranslationJob(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/strings
            Responses:
                200 : OK
                404 : Job not found error
            details :  https://api-reference.smartling.com/#operation/getStringsForTranslationJob
        """
        translationJobUid=self.test_job_uid
        res, status = self.jobs_api.getStringsForTranslationJob(translationJobUid=translationJobUid)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getStringsForTranslationJob', 'OK')


    def checkFindJobsByStrings(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/find-jobs-by-strings
            Responses:
                200 : OK
                400 : Validation error response
            details :  https://api-reference.smartling.com/#operation/findJobsByStrings
        """
        hashcodes=[]
        localeIds=[self.MY_LOCALE,]
        res, status = self.jobs_api.findJobsByStrings(hashcodes=hashcodes, localeIds=localeIds)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('findJobsByStrings', 'OK')


    def checkSearchForJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/search
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/searchForJob
        """
        fileUris=[]
        hashcodes=[]
        translationJobUids=[self.test_job_uid]
        res, status = self.jobs_api.searchForJob(fileUris=fileUris, hashcodes=hashcodes, translationJobUids=translationJobUids)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('searchForJob', 'OK')


    def checkCancelJob(self):
        """
            method  :  POST
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}/cancel
            Responses:
                200 : OK
                202 : ACCEPTED
                400 : Validation error when cancelling a job
                401 : Authentication error
            details :  https://api-reference.smartling.com/#operation/cancelJob
        """
        translationJobUid=self.test_job_uid
        reason='test reason'
        res, status = self.jobs_api.cancelJob(translationJobUid=translationJobUid, reason=reason)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('cancelJob', 'OK')


    def checkDeleteJob(self):
        """
            method  :  DELETE
            api url :  /jobs-api/v3/projects/{projectId}/jobs/{translationJobUid}
            Responses:
                200 : OK
                400 : Validation error when deleting a job
                401 : Authentication error
            details :  https://api-reference.smartling.com/#operation/deleteJob
        """
        translationJobUid=self.test_job_uid
        res, status = self.jobs_api.deleteJob(translationJobUid=translationJobUid)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('deleteJob', 'OK')


    def checkGetProjectCustomFields(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/projects/{projectId}/custom-fields
            Responses:
                200 : OK
                404 : Not found error
            details :  https://api-reference.smartling.com/#operation/getProjectCustomFields
        """
        res, status = self.jobs_api.getProjectCustomFields()

        print('getProjectCustomFields', 'OK')


    def checkGetAccountCustomFields(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/accounts/{accountUid}/custom-fields
            Responses:
                200 : OK
                404 : Not found error
            details :  https://api-reference.smartling.com/#operation/getAccountCustomFields
        """
        accountUid=self.MY_ACCOUNT_UID
        res, status = self.jobs_api.getAccountCustomFields(accountUid=self.MY_ACCOUNT_UID)

        assert_equal(True, status in [200,202])
        assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])
        print('getAccountCustomFields', 'OK')


    def checkGetJobsByAccount(self):
        """
            method  :  GET
            api url :  /jobs-api/v3/accounts/{accountUid}/jobs
            as curl :  curl -H "Authorization: Bearer $smartlingToken" https://api.smartling.com/jobs-api/v3/accounts/$smartlingAccountId/jobs
            Responses:
                200 : OK
            details :  https://api-reference.smartling.com/#operation/getJobsByAccount
        """
        accountUid=self.MY_ACCOUNT_UID
        res, status = self.jobs_api.getJobsByAccount(accountUid=self.MY_ACCOUNT_UID)

        print('getJobsByAccount', 'OK')



def example():
    t = testJobsApi()
    t.setUp()
    t.checkAssignCustomFieldsToProject()
    t.checkAddJob()
    t.checkAddLocaleToJob()
    t.checkAddStringsToJob()
    t.checkAddFileToJob()
    t.checkGetJobFilesList()
    t.checkGetJobFileProgress()
    t.checkAuthorizeJob()
    t.checkModifyScheduleItemsForTranslationJob()
    t.checkCreateCustomField()
    t.checkUpdateCustomField()
    t.checkRemoveFileFromJob()
    t.checkRemoveStringsFromJob()
    t.checkGetJobLastCompletionDatesPerLocale()
    t.checkFindScheduleForTranslationJob()
    t.checkRemoveLocaleFromJob()
    t.checkGetJobsByProject()
    t.checkUpdateJob()
    t.checkGetJobProgress()
    t.checkGetJobDetails()
    t.checkGetStringsForTranslationJob()
    t.checkFindJobsByStrings()
    t.checkSearchForJob()
    t.checkCancelJob()
    t.checkDeleteJob()
    t.checkGetProjectCustomFields()
    t.checkGetAccountCustomFields()
    t.checkGetJobsByAccount()
    # not covered by tests #
    '''
    closeJob
    getJobAsyncProcessStatus
    '''
    t.tearDown()

if __name__ == '__main__':
    example()



#!/usr/bin/python
# -*- coding: utf-8 -*-


""" Copyright 2012-2021 Smartling, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this work except in compliance with the License.
 * You may obtain a copy of the License in the LICENSE file, or at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limit
"""

import json
import importlib
from builder.Parameters import ApiCore, Parameter, MultipartProperty


class Method(ApiCore):
    indent = '    '
    indent2 = indent*2
    indent3 = indent*3
    indent4 = indent*4

    def __init__(self, apiName, apiNameUnderscore, path, method, descriptionDict, swaggerDict):
        ApiCore.__init__(self, swaggerDict)
        self.operationId = ''
        self.type = ''
        self.multipartParameters = []
        self.requestBody = ''
        for name in ['summary', 'description', 'tags', 'operationId', 'responses', 'x-code-samples', 'requestBody']:
            setattr(self, name, descriptionDict.get(name, None))
        self.apiName = apiName
        self.apiNameUnderscore = apiNameUnderscore
        self.method = method
        self.path = path
        self.parameters = []

        for p in descriptionDict['parameters']:
            if 'projectId' == p['name']:
                continue  # We have projectId as api class member
            parameter = Parameter(p, swaggerDict)
            self.parameters .append(parameter)

        self.needMultipart = False
        self.isJson = False
        self.hasDirectives = False
        self.getMultipartProps()

    def build(self):
        rows = [
            self.buildName(),
            self.buildDoc()
        ]

        if self.needMultipart:
            rows.append(self.buildMultipart())
        else:
            rows.append(self.buildBody())

        return '\n'.join(rows)

    def buildExample(self):
        rows = [
            self.buildTestName(),
            self.buildDoc(),
            self.buildTestBody(),
            ''
        ]
        joined = '\n'.join(rows)
        joined = joined.replace(self.indent2 + '\n', '\n')  # Remove whitespaces for separator line
        return joined

    def buildTestName(self):
        return ''.join([
            self.indent,
            'def check',
            self.operationId[0].capitalize() + self.operationId[1:],
            '(self):',
            ]
        )

    def buildName(self):
        return ''.join([
            self.indent,
            'def ',
            self.operationId,
            '(self',
            self.buildParams(),
            ', **kwargs):',
        ]
        )

    def buildParams(self):
        result = ''
        joined = self.parameters + self.multipartParameters
        self.rearrangeRequired(joined)
        if joined:
            result += ", " + ", ".join(x.getParamForName() for x in joined)
        if self.hasDirectives:
            result += ', directives={}'
        return result

    def buildDoc(self):
        commentMarker = self.indent2 + '"""'
        docLines = [
            commentMarker,
            self.indent3 + 'method  :  ' + self.method.upper(),
            self.indent3 + 'api url :  ' + self.path,
        ]
        curl_example = self.getCurlExample()
        if curl_example:
            docLines.append(curl_example)

        nested = self.listNestedValues()
        if nested:
            docLines.append(nested)
        docLines = self.getResponsesDescription(docLines)
        details = self.indent3 + 'details :  https://api-reference.smartling.com/#operation/' + self.operationId
        docLines.append(details)
        docLines.append(commentMarker)

        return '\n'.join(docLines)

    def getResponsesDescription(self, docLines):
        result = []
        responses = getattr(self, 'responses', {})
        for code, codeDict in responses.items():
            descr = codeDict.get('description', '')
            if descr:
                result.append(self.indent2 + '%s : %s' % (code, descr))
        if result:
            result.insert(0, self.indent + "Responses:")
            responses = self.joinWithIndent(result, self.indent2)
            docLines.append(responses)
        return docLines

    def getCurlExample(self):
        result = []
        samples = getattr(self, 'x-code-samples', [])
        if not samples:
            return ''

        for d in samples:
            result.append(self.indent + 'as curl :  ' + d['source'].replace('\n', ''))
        return self.joinWithIndent(result, self.indent2)

    def getPropertyDescription(self, prop):
        prop_dict = {}
        for m in prop.prop_list:
            prop_dict[m._name] = getattr(m, '_example', '') or m.getDefault()

        result = [self.indent + 'Parameters example:']
        hdr = self.indent + '%s: ' % prop._name
        dumped = json.dumps(prop_dict, indent=16)
        dumped = dumped.replace('}', self.indent4+'}')
        result.append(hdr + dumped)
        return result

    def listNestedValues(self):
        result = []
        for prop in self.multipartParameters:
            if not getattr(prop, 'prop_list', None):
                continue
            result += self.getPropertyDescription(prop)
        if result:
            return self.joinWithIndent(result, self.indent2)
        return ''

    def joinWithIndent(self, lst, indent):
        newlinePlusIndent = '\n'+indent
        return indent + newlinePlusIndent.join(lst)

    def buildPathParamsStr(self):
        pathParameters = [x for x in self.parameters if x._in == 'path']
        if not pathParameters:
            return ''
        pthArgs = ['%s=%s' % (x._name, x._name) for x in pathParameters]
        return ', ' + ', '.join(pthArgs)

    def buildBody(self):
        bodyLines = []

        valuesToPass = "kw"
        bodyLines.append('kw = {')

        kw_params = [x for x in self.parameters if x._in == 'query']
        for p in kw_params:
            bodyLines.append(self.indent + "'%s':%s," % (p._name, p._name))
        for m in self.multipartParameters:
            if m.isRequestBody:
                valuesToPass = m._name
                continue
            if 'binary' == m._format:
                raise Exception("Incompatible parameter format for command")
            bodyLines.append(self.indent + "'%s':%s," % (m._name, m._name))
        bodyLines.append('}')
        bodyLines.append('kw.update(kwargs)')
        bodyLines.append("url = self.urlHelper.getUrl('%s'%s, **kwargs)" % (self.path, self.buildPathParamsStr()))
        cmd = "response, status = self.command('%s', url, %s)" % (self.method.upper(), valuesToPass)
        if self.method.upper() in ('POST', 'PUT') and self.isJson:
            cmd = "response, status = self.commandJson('%s', url, %s)" % (self.method.upper(), valuesToPass)
        bodyLines.append(cmd)
        bodyLines.append("return response, status")

        return self.joinWithIndent(bodyLines, self.indent2)

    def buildTestBody(self):
        bodyLines = []

        parameters = []
        initialisers = {}

        testDataModule = importlib.import_module('testdata.'+self.apiName+'TestData')
        decorators = getattr(testDataModule, 'testDecorators')

        jobsTestData = None
        if self.operationId in decorators.keys():
            jobsTestData = decorators[self.operationId]
            initialisers = jobsTestData.fields
            for line in jobsTestData.preCalls:
                bodyLines.append(line)
        for p in self.parameters + self.multipartParameters:
            if p._required or p._name in initialisers:
                parameters.append(p.getParamForMethodCall())

        kwParams = [x for x in self.parameters if x._in == 'query' or x._in == 'path']
        for p in kwParams + self.multipartParameters:
            if p._required or p._name in initialisers:
                bodyLines.append(p.getParamForMethodCall(initialisers))

        callParams = ', '.join(parameters)
        bodyLines.append('res, status = self.%s_api.%s(%s)' % (self.apiNameUnderscore, self.operationId, callParams))
        bodyLines.append('')

        if jobsTestData and jobsTestData.customTestCheck:
            bodyLines += jobsTestData.customTestCheck.split('\n')

        if jobsTestData and jobsTestData.is_apiv2_response:
            bodyLines.append('assert_equal(True, status in [200,202])')
            bodyLines.append('assert_equal(True, res.code in [self.CODE_SUCCESS_TOKEN, self.ACCEPTED_TOKEN])')
        bodyLines.append("print('%s', 'OK')" % self.operationId)
        if jobsTestData:
            for line in jobsTestData.postCalls:
                bodyLines.append(line)

        return self.joinWithIndent(bodyLines, self.indent2)

    def listProperty(self, name):
        if not name:
            raise Exception("Can't determine property name")
        mp = MultipartProperty(name, {'type': 'array'}, self.swaggerDict)
        mp.setRequired()
        mp.isRequestBody = True
        self.multipartParameters.insert(0, mp)

    def getMultipartProps(self):
        self.multipartParameters = []
        if not self.requestBody:
            return
        self.resolveRequestBodyRef()
        self.type = list(self.requestBody['content'].keys())[0]
        if 'application/json' == self.type:
            self.isJson = True

        schema = self.requestBody['content'][self.type]['schema']

        refname = ''
        if '$ref' == list(schema.keys())[0]:
            schema, refname = self.resolveRef(schema['$ref'])

        props = schema.get('properties', None)
        if props is None:
            if 'array' == schema['type']:
                self.listProperty(refname)
                return
            else:
                print (schema)

        self.multipartParameters = self.parseProperties(props)

        for req in schema.get('required', []):
            for mp in self.multipartParameters:
                if req == mp._name:
                    mp.setRequired()

        self.rearrangeRequired(self.multipartParameters)

    def resolveRequestBodyRef(self):
        for key in self.requestBody:
            if '$ref' == key:
                resolved, refname = self.resolveRef(self.requestBody['$ref'])
                self.requestBody = resolved
                return

    def rearrangeRequired(self, params):
        needRearrangeIndex = []
        hasOptional = False
        positionToInsert = 0
        for p in params:
            if not p._required:
                hasOptional = True
                if not hasOptional:
                    positionToInsert = params.index(p)
                continue
            if hasOptional:
                needRearrangeIndex.append(params.index(p))
        for idx in needRearrangeIndex:
            p = params[idx]
            del params[idx]
            params.insert(positionToInsert, p)
            positionToInsert += 1

    def parseProperties(self, props):
        propList = []
        for k in props.keys():
            propDict = props[k]
            if k.startswith('smartling.'):
                self.hasDirectives = True
                continue

            mp = MultipartProperty(k, propDict, self.swaggerDict)
            propList.append(mp)

            if 'application/json' == self.type:
                if mp._description and 'required' in mp._description:
                    mp.setRequired()

            if propDict.get('properties', None):
                mp.prop_list = self.parseProperties(propDict['properties'])

            if mp._format == 'binary':
                self.needMultipart = True
        return propList

    def buildMultipart(self):
        result = [
            'kw = {',
        ]
        kw_params = [x for x in self.parameters if x._in == 'query']
        for p in kw_params:
            result.append(self.indent + "'%s':%s," % (p._name, p._name))
        for m in self.multipartParameters:
            if 'binary' == m._format:
                result.append(self.indent + "'%s':self.processFile(%s)," % (m._name, m._name))
            elif not 'directives' == m._name:
                result.append(self.indent + "'%s':%s," % (m._name, m._name))
        result.append('}')

        result += self.addDirectives()
        result.append("url = self.urlHelper.getUrl('%s'%s)" % (self.path, self.buildPathParamsStr()))
        result.append("return self.uploadMultipart(url, kw)")

        return self.joinWithIndent(result, self.indent2)

    def addDirectives(self):
        if self.hasDirectives:
            return ["self.addLibIdDirective(kw)",
                    'self.processDirectives(kw, directives)']
        else:
            return []



#!/usr/bin/python
# -*- coding: utf-8 -*-


""" Copyright 2012-2021 Smartling, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this work except in compliance with the License.
 * You may obtain a copy of the License in the LICENSE file, or at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limit
"""

from builder.Parameters import Code
from builder.ExampleData import TestData

testsOrder = [
    'uploadNewVisualContext',
    'getVisualContextsListByProject',
    'getVisualContextInfo',
    'downloadVisualContextFileContent',
    'runAutomaticContextMatching',
    'uploadAndMatchVisualContext',
    'getAsyncContextMatchResults',
    'createStringToContextBindings',
    'getBindings',
    'deleteBindings',
    'deleteVisualContext',
]

extraInitializations = '''
'''

testEnvironment = 'stg'

videoUrl = 'https://www.youtube.com/watch?v=0lJykuiS_9s'

stringsAdded = [
    {"hashcode":"ede6083ebd2594ca4e557612aaa05b2e","stringText":"Usability Testing","parsedStringText":"Usability Testing"},
    {"hashcode":"4f25feab674accf572433f22dc516e2e","stringText":"Service","parsedStringText":"Service"},
    {"hashcode":"b60df7845b7a3755fa00a833a31fa91e","stringText":"Design for Context","parsedStringText":"Design for Context"}
]

testDecorators = {
    'uploadNewVisualContext':TestData(
        {
            'name' : videoUrl
        },
        [],
        [
            'self.context_uid = res.data.contextUid',
        ],
        customTestCheck='''
assert_equal(res.data.contextType, 'VIDEO')
assert_equal(res.data.name, '%s')

content = smartlingApiSdk.__path__[0]+'/resources/ctx_api_test.png'
res_img, status = self.context_api.uploadNewVisualContext(content=content)
self.context_uid_img = res_img.data.contextUid
''' % videoUrl
    ),

    'getVisualContextsListByProject':
        TestData({}, customTestCheck='''
print('Total context count:',len(res.data.items))
assert_equal(len(res.data.items) > 0, True)
'''
                 ),


    'getVisualContextInfo' : TestData(
        {
            'contextUid': Code('self.context_uid'),
        },
        customTestCheck='''
assert_equal(res.data.contextType, 'VIDEO')
assert_equal(res.data.name, '%s')
''' % videoUrl
    ),


    'downloadVisualContextFileContent' : TestData(
        {
            'contextUid': Code('self.context_uid_img'),
        },
        customTestCheck='''
assert_equal(86324, len(res)) #empty for video context
''',
        isApiV2Response= False,
    ),


    'runAutomaticContextMatching' : TestData(
        {
            'contextUid': Code('self.context_uid_img'),
            'contentFileUri' : '',
            'stringHashcodes' : '',
            'overrideContextOlderThanDays' : 1,
        },
        customTestCheck='''
self.match_id = res.data.matchId
'''
    ),

    'getAsyncContextMatchResults' : TestData(
        {
            'matchId': Code('self.match_id_upl_n_match'),
        },
        customTestCheck='''
'''
    ),


    'deleteVisualContext' : TestData(
        {
            'contextUid': Code('self.context_uid'),
        },
        customTestCheck='''
res2, status = self.context_api.deleteVisualContext(contextUid=self.context_uid_img)
'''
    ),

    'uploadAndMatchVisualContext' : TestData(
        {
            'content' : Code("smartlingApiSdk.__path__[0]+'/resources/ctx_api_test.png'"),
        },
        customTestCheck='''
self.match_id_upl_n_match = res.data.matchId
'''
    ),

    'createStringToContextBindings': TestData(
        {
            'bindings': Code("[{'contextUid': self.context_uid, 'stringHashcode': 'ede6083ebd2594ca4e557612aaa05b2e'},\n             {'contextUid': self.context_uid_img, 'stringHashcode': '4f25feab674accf572433f22dc516e2e'}]"),
        },
        customTestCheck='''
assert_equal(res.data.errors['totalCount'], 0)
assert_equal(res.data.created['totalCount'], 2)
items = res.data.created['items']
self.binding_uno = items[0]['bindingUid']
self.binding_dos = items[1]['bindingUid']
'''
    ),

    'getBindings': TestData(
        {
            'stringHashcodes': ['ede6083ebd2594ca4e557612aaa05b2e','4f25feab674accf572433f22dc516e2e'],
            'contentFileUri' : '',
            'contextUid' : '',
            'bindingUids' : [],
        },
        customTestCheck='''
print('Total bindings count:',len(res.data.items))
assert_equal(len(res.data.items), 2)
'''
    ),

    'deleteBindings': TestData(
        {
            'stringHashcodes': [],
            'contentFileUri' : '',
            'contextUid' : '',
            'bindingUids' : Code('[self.binding_uno, self.binding_dos]'),
        },
        customTestCheck=''
    ),
}



#!/usr/bin/python
# -*- coding: utf-8 -*-


""" Copyright 2012-2021 Smartling, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this work except in compliance with the License.
 * You may obtain a copy of the License in the LICENSE file, or at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limit
 """

import importlib
from builder.Parameters import MultipartProperty
from builder.Method import Method
from builder.ExampleData import EXAMPLE_HEADER, EXAMPLE_FOOTER, TESTS_FOOTER, COPYRIGHT_HEADER


class ApiSource():
    def __init__(self, fullName, apiName):
        self.fullName = fullName
        self.methods = []
        self.apiName = apiName
        self.apiNameUnderscore = fullName.replace(' ', '_').replace('&', '').replace('__', '_').lower()

    def collectMethods(self, swaggerDict):
        toBuild = []
        for k, v in swaggerDict['paths'].items():
            for method, descr in v.items():
                if method == '$ref':
                    continue
                if toBuild and descr['operationId'] not in toBuild:  # Debug build for specific methods only
                    continue
                if self.fullName in descr['tags']:
                    m = Method(self.apiName, self.apiNameUnderscore, k, method, descr, swaggerDict)
                    self.patchMethods(descr, m, swaggerDict)
                    self.methods.append(m)

    def patchMethods(self, descr, m, swaggerDict):
        if descr['operationId'] == 'exportFileTranslations':
            propDict = {
                "type": "string",
                "format": "binary",
                "description": "The file contents to upload."
            }
            mp = MultipartProperty('file', propDict, swaggerDict)
            mp.setRequired()
            m.needMultipart = True
            m.multipartParameters.insert(0, mp)
        if descr['operationId'] in ('getAllSourceStringsByProject'):
            m.method = 'post'
            m.isJson = True
        if descr['operationId'] in ('addStringsToProject'):
            for p in m.parameters+m.multipartParameters:
                if p._name == 'namespace':
                    p._default = 'smartling.strings-api.default.namespace'
        if descr['operationId'] in ('getAllTranslationsByProject'):
            m.method = 'post'
            m.isJson = True
            if m.parameters[0]._name == 'hashcodes':
                p = m.parameters[0]
                del (m.parameters[0])
                m.parameters.insert(1, p)

    def build(self):

        header = COPYRIGHT_HEADER + '''

from smartlingApiSdk.ApiV2 import ApiV2

class %sApi(ApiV2):

    def __init__(self, userIdentifier, userSecret, projectId, proxySettings=None, permanentHeaders={}, env='prod'):
        ApiV2.__init__(self, userIdentifier, userSecret, projectId, proxySettings, permanentHeaders=permanentHeaders, env=env)
''' % self.apiName
        rows = header.split('\n')
        for m in self.methods[:]:
            built = m.build()
            if built:
                rows.append(built)
                rows.append('')
                rows.append('')
        return '\n'.join(rows)

    def methodByName(self, name):
        for m in self.methods:
            if m.operationId == name:
                return m
        raise Exception("Can't find method:"+name)

    def buildExampleHeader(self):
        # Do dynamic imports based on apy_name
        testDataModule = importlib.import_module('testdata.'+self.apiName+'TestData')
        imports = getattr(testDataModule, 'imports', '')
        extraInitializations = getattr(testDataModule, 'extraInitializations')
        tearDown = getattr(testDataModule, 'tearDown', '')
        testsOrder = getattr(testDataModule, 'testsOrder')
        testEnvironment = getattr(testDataModule, 'testEnvironment', 'prod')

        hdr = EXAMPLE_HEADER
        if 'stg' == testEnvironment:
            hdr = hdr.replace('Credentials()', "Credentials('stg')")
            hdr = hdr.replace('proxySettings)', "proxySettings, env='stg')")
        tearDownMarker = '        print("tearDown", "OK")'
        hdr = hdr.replace(tearDownMarker, tearDown+tearDownMarker)
        hdr = hdr.replace("{EXTRA_IMPORTS}\n", imports)
        hdr += extraInitializations

        return hdr, testsOrder

    def buildTestOrExample(self, footer, indent):
        rows = []

        hdr, testsOrder = self.buildExampleHeader()

        apiNameApi = self.apiName + "Api"
        hdr = hdr.replace('{API_NAME}', apiNameApi)
        hdr = hdr.replace('{api_name}', self.apiNameUnderscore)
        ftr = footer.replace('{API_NAME}', apiNameApi)

        notTestedCalls = [m.operationId for m in self.methods]
        notTestedCalls.insert(0, "'''")
        notTestedCalls.insert(0, '# not covered by tests #')
        rows.append(hdr)
        testCalls = []

        for name in testsOrder:
            m = self.methodByName(name)
            if name in notTestedCalls:  # Test may occur twice in tests list
                notTestedCalls.remove(name)

            built = m.buildExample()
            capitalized = m.operationId[0].capitalize() + m.operationId[1:]

            testCalls.append('t.check%s()' % capitalized)

            if built:
                rows.append(built)
                rows.append('')

        notTestedCalls.append("'''")
        if len(notTestedCalls) > 3:
            testCalls += notTestedCalls

        newlineWithIndent = '\n' + '    ' * indent
        rows.append(ftr % newlineWithIndent.join(testCalls))
        return '\n'.join(rows)

    def buildExample(self):
        return self.buildTestOrExample(EXAMPLE_FOOTER, indent=1)

    def buildTest(self):
        return self.buildTestOrExample(TESTS_FOOTER, indent=2)


